/**************************************
 * $Author: kaneda $
 * $Date: 2003/02/12 12:45:18 $
 **************************************/

/* ph_vp.h */

#ifndef __PH_VP_H__
#define __PH_VP_H__

#include "ph_util.h"


/* Structures */

/* PH_MAX_VP = (long long)(1LL << 63) - 1LL   (tmporary address $B$b4^$a$?>e8B(B) */
/* ph_vp.c */

#ifndef __PH_WIN32__

#define PH_INVALID_VP		-1LL
#define PH_MIN_VP		0LL
#define PH_MAX_VP		9223372036854775807LL
#define PHOENIX_VP_LIMIT 	(1LL << 62)
#define PHOENIX_MAX_VP 	 	((1LL << 62) - 1LL)
typedef long long 		ph_vp_t;
#define ph_get_ph_vp_t(a)	a ## LL
#define PH_LLD 			"%lld"

#else

#define PH_INVALID_VP		-1
#define PH_MIN_VP		0
#define PH_MAX_VP		9223372036854775807
#define PHOENIX_VP_LIMIT 	4611686018427387904
#define PHOENIX_MAX_VP 	 	4611686018427387903
typedef __int64 		ph_vp_t;
#define ph_get_ph_vp_t(a)	a
#define PH_LLD 			"%I64d"

#endif


struct ph_vp_range {
  ph_vp_t			lower, upper;
};

typedef struct ph_vp_range 	* ph_vp_range_t;


PH_DEFINE_LIST_STRUCTURE(ph_vp_range) 


typedef struct ph_vp_range_list 	ph_vp_set;
typedef struct ph_vp_range_list		* ph_vp_set_t;


struct ph_vp_set_sync {
  ph_vp_set_t			vp_set;
  pthread_mutex_t		mp;
};

typedef struct ph_vp_set_sync 	* ph_vp_set_sync_t;


ph_vp_range_t ph_vp_range_new(ph_vp_t lower, ph_vp_t upper);
void ph_vp_range_dispose(ph_vp_range_t vp_range);
ph_vp_range_t ph_vp_range_dup(ph_vp_range_t vp_range);
int ph_vp_range_is_sub(ph_vp_range_t p, ph_vp_range_t q);
int ph_vp_range_is_intersected(ph_vp_range_t p, ph_vp_range_t q);
int ph_vp_range_is_exclusive(ph_vp_range_t p, ph_vp_range_t q);
int ph_vp_range_is_member(ph_vp_range_t p, ph_vp_t vp);


PH_DEFINE_LIST_FUNC_PROTOTYPE(ph_vp_range) 


ph_vp_set_t ph_vp_set_new();
void ph_vp_set_dispose(ph_vp_set_t vp_set);
void ph_vp_set_print(ph_vp_set_t vp_set);

ph_vp_set_t ph_vp_set_dup(ph_vp_set_t vp_set);
int ph_vp_set_is_empty(ph_vp_set_t vp_set);
int ph_vp_set_is_sub(ph_vp_set_t p, ph_vp_set_t q);
int ph_vp_set_is_equal(ph_vp_set_t p, ph_vp_set_t q);
int ph_vp_set_is_intersected(ph_vp_set_t p, ph_vp_set_t q);
int ph_vp_set_is_exclusive(ph_vp_set_t p, ph_vp_set_t q);
int ph_vp_set_is_member(ph_vp_set_t vp_set, ph_vp_t vp);

ph_vp_set_t ph_vp_set_make_empty();
ph_vp_set_t ph_vp_set_make_singleton(ph_vp_t vp);
ph_vp_set_t ph_vp_set_make_range(ph_vp_t l, ph_vp_t u);

ph_vp_set_t ph_vp_set_make_intersection(ph_vp_set_t p, ph_vp_set_t q);
ph_vp_set_t ph_vp_set_make_union(ph_vp_set_t p, ph_vp_set_t q);
ph_vp_set_t ph_vp_set_make_diff(ph_vp_set_t p, ph_vp_set_t q);
ph_vp_t ph_vp_set_get_min_elem(ph_vp_set_t vp_set);
ph_vp_t ph_vp_set_get_max_elem(ph_vp_set_t vp_set);
ph_vp_t ph_vp_set_get_any_elem(ph_vp_set_t vp_set);

void ph_vp_set_add_range(ph_vp_set_t * vp_set_p, ph_vp_t lower, ph_vp_t upper);
void ph_vp_set_remove_range(ph_vp_set_t * vp_set_p, ph_vp_t lower, ph_vp_t upper);
void ph_vp_set_vps2str(ph_vp_set_t vp_set, char * s);
ph_vp_set_t ph_vp_set_str2vps(char * s);
void ph_vp_set_vps2str_for_print(ph_vp_set_t vp_set, char * s);



ph_vp_set_sync_t ph_vp_set_sync_new();
void ph_vp_set_sync_dispose(ph_vp_set_sync_t vp_set_sync);
void ph_vp_set_sync_assume(ph_vp_set_sync_t p, ph_vp_set_t q);
void ph_vp_set_sync_release(ph_vp_set_sync_t p, ph_vp_set_t q);
ph_vp_set_t ph_vp_set_sync_dup(ph_vp_set_sync_t vp_set_sync);
int ph_vp_set_sync_is_member(ph_vp_set_sync_t vp_set_sync, ph_vp_t vp);
ph_vp_t ph_vp_set_sync_get_max_elem(ph_vp_set_sync_t vp_set_sync);
#endif


