/**************************************
 * $Author: kaneda $
 * $Date: 2002/10/16 03:00:03 $
 **************************************/

/* sock.c */
#include "ph_util.h"
#include "ph_pt2pt.h"



static ssize_t sendn(int fd, void * buf, size_t len) {
  ssize_t n = 0, m;
 
  while (((m = send(fd, (char *)buf+n, len - n, 0)) > 0) && ((n += m) < len)) {
  }

  return n;
}

static ssize_t writen(int fd, void * buf, size_t len) {
  ssize_t n = 0, m;
 
  while (((m = write(fd, (char *)buf+n, len - n)) > 0) && ((n += m) < len)) {
  }

  return n;
}


static send_ready() {
  writen(1, "OK", 3);
}


static void initialize_select(fd_set * r_fdset_p, fd_set * w_fdset_p, fd_set * e_fdset_p, int fd) {
  FD_ZERO(r_fdset_p);
  FD_ZERO(w_fdset_p);
  FD_ZERO(e_fdset_p);

  FD_SET(0, r_fdset_p);
  FD_SET(0, e_fdset_p);

  FD_SET(fd, r_fdset_p);
  FD_SET(fd, e_fdset_p);
}

static int handle_io(fd_set * r_fdset_p, fd_set * w_fdset_p, fd_set * e_fdset_p, int fd) {
  char s[PH_BUFSIZE];
  ssize_t n;

  // $BC1=c$K(B forward $B$9$k$@$1$G$-$A$s$HF0:n$9$k(B?
  if (FD_ISSET(0, r_fdset_p)) {
    ph_mzero(s, PH_BUFSIZE);
    if ((n = read(0, s, PH_BUFSIZE)) <= 0) {
      return 0;
    }
    //fprintf(stderr, "\n\nfrom [0] to [%d]  (len=%d)\n", fd, n);
    sendn(fd, s, n);
    //fprintf(stderr, "\"%s\"\n\n", s);
  }
  
  if (FD_ISSET(fd, r_fdset_p)) {
    ph_mzero(s, PH_BUFSIZE);
    if ((n = recv(fd, s, PH_BUFSIZE, 0)) <= 0) {
      return 0;
    }
    //fprintf(stderr, "\n\nfrom [%d] to [1]  (len=%d)\n", fd, n);
    //fprintf(stderr, "\"%s\"\n\n", s);
    writen(1, s, n);
  }


  /* Can this program forward messages correctly? */
  if ((FD_ISSET(0, e_fdset_p)) || (FD_ISSET(fd, e_fdset_p))) {
    return 0;
  }  
  return 1;
}


static void forward(char * host, char * service) {
  int fd;
  ph_addr_t addr;
  ph_tcp_attr_t attr;

  addr = ph_addr_new(host, service, tcp, NULL);

  if ((attr = (ph_tcp_attr_t)ph_tcp_attr_connect(addr)) == NULL) {
    // connect failure
    return;
  }

  send_ready();

  while (1) {
    fd_set r_fdset, w_fdset, e_fdset;

    initialize_select(&r_fdset, &w_fdset, &e_fdset, attr->fd);  
    ph_select(PH_MAX_FD, &r_fdset, &w_fdset, &e_fdset, NULL);
    if (handle_io(&r_fdset, &w_fdset, &e_fdset, attr->fd) == 0) {
      break;
    }
  }
  
  
  ph_tcp_attr_close((void *)attr);
  ph_tcp_attr_dispose((void *)attr);
  ph_addr_dispose(addr);
}


int main(int argn, char ** argv, char ** envp) {

  if (argn != 3) {
    printf("Usage: %s host service\n", argv[0]);
    return 1;
  }

 
  forward(argv[1], argv[2]);
  return 0;
}












