/**************************************
 * $Author: kaneda $
 * $Date: 2002/12/20 10:14:40 $
 **************************************/

/* util.h */

#ifndef __UTIL_H__
#define __UTIL_H__
 

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <stdarg.h> /* ANSI C header file */
#include <assert.h>
#include <errno.h>
#include <signal.h>
#include <fcntl.h>
#include <ctype.h>
#include <limits.h>
#include <sys/types.h>	/* basic system data types */
#include <sys/stat.h>
#include <unistd.h>
#include <strings.h>
#include <netdb.h>
#include <pthread.h>
#include <sys/time.h>
#include <sys/file.h>
#include <sys/socket.h>	/* basic socket definitions */
#include <sys/wait.h>
#include <sys/un.h>		/* for Unix domain socket */
#include <netinet/in.h>	/* sockaddr_in{} and other Internet defns */
#include <netinet/in_systm.h>
#include <netinet/ip.h>
#include <netinet/tcp.h>
#include <arpa/inet.h>	/* inet(3) functions */

#include "phoenix.h"

#ifndef _REENTRANT
#define _REENTRANT
#endif


#ifndef INADDR_NONE
#define INADDR_NONE	0xffffffff	/* should have beeein in <netinet/in.h> */
#endif


/* Macro Values */
enum {
  MAX_FD 	= 1024,
  LISTENQ	= 1024,	/* nd argument to listen() */

  MAX_LINE	= 4096,	/* max text line length */
  BUFSIZE	= 8192
};


/* Basic Types */
typedef void Sigfunc(int);

/* Basic Types */
#ifndef HAVE_GETADDRINFO
#define socklen_t int
#endif



/* Structures */
/* ph_cell_t */
struct cell {
  void 			* self;
  struct cell	 	* next;
};

typedef struct cell	* cell_t;

/* ph_list_t */
struct list {
  cell_t		head;
  cell_t 		tail;
  int			nelm;

  int			is_sync;		/* if 1 then insert/delete of item is synchronized */

  pthread_mutex_t	mp;			/* synchronize insertion/detetion of a cell */
  pthread_cond_t	i_cond;			/* It is signaled when the new item is inserted to the list. */
  pthread_cond_t	d_cond;			/* It is signaled when the item is deleted. */
};

typedef struct list	* list_t;

typedef void (*func1)(void *);
typedef int (*func2)(void *, void *);
typedef void (*func3)(void *, void *);
typedef int (*func4)(void *, void *, void *);


/* Public Global Variables */


/* this is defined in error.c */
extern int daemon_proc;



/* Public Function Prototypes */

/* Following functions are defined in debug.c */
void debug_set_mode(int mode);
int debug_get_mode();
void debug_print(char * fmt, ...);
void debug_print2(char * fmt, ...);
void debug_print3(char * fmt, ...);
void debug_print4(char * fmt, ...);
void debug_print_message(pthread_mutex_t mp, char * buf, int len);

/* Following functions are defined in error.c */
void err_ret(const char * fmt, ...);
void err_sys(const char * fmt, ...);
void err_dump(const char * fmt, ...);
void err_msg(const char * fmt, ...);
void err_quit(const char * fmt, ...);

/* Following functions are defined in wrap_unix.c */
void Free(void *ptr);
void * Malloc(size_t size);
void * Calloc(size_t n, size_t size);
void Close(int fd);
void Dup(int fd);
void Dup2(int fd1, int fd2);
int Fcntl(int fd, int cmd, int arg);
void Gettimeofday(struct timeval *tv, void *foo);
int Ioctl(int fd, int request, void *arg);
pid_t Fork(void);
char * Getenv(const char *name);
void Mkstemp(char *temp);
int Open(const char *pathname, int oflag, mode_t mode);
void Pipe(int *fds);
ssize_t Read(int fd, void *ptr, size_t nbytes);
void Sigaddset(sigset_t *set, int signo);
void Sigdelset(sigset_t *set, int signo);
void Sigemptyset(sigset_t *set);
void Sigfillset(sigset_t *set);
int Sigismember(const sigset_t *set, int signo);
void Sigpending(sigset_t *set); 
void Sigprocmask(int how, const sigset_t *set, sigset_t *oset);
long Sysconf(int name);
void Unlink(const char *pathname);
pid_t Wait(int *iptr);
pid_t Waitpid(pid_t pid, int *iptr, int options);
void Write(int fd, void *ptr, size_t nbytes);
FILE * Fopen(const char * filename, const char * mode);
void Fclose(FILE * fp);
Sigfunc * Signal(int signo, Sigfunc * func);


/* Following functions are defined in wrap_sock.c */
int Accept(int fd, struct sockaddr *sa, socklen_t *salenptr);
void Bind(int fd, const struct sockaddr *sa, socklen_t salen);
void Connect(int fd, const struct sockaddr *sa, socklen_t salen);
void Getpeername(int fd, struct sockaddr *sa, socklen_t *salenptr);
void Getsockname(int fd, struct sockaddr *sa, socklen_t *salenptr) ;
void Getsockopt(int fd, int level, int optname, void *optval, socklen_t *optlenptr);
void Listen(int fd, int backlog);
int Select(int nfds, fd_set *readfds, fd_set *writefds, fd_set *exceptfds, struct timeval *timeout);
void Setsockopt(int fd, int level, int optname, const void *optval, socklen_t optlen);
void Shutdown(int fd, int how);
int Socket(int family, int type, int protocol);
void Socketpair(int family, int type, int protocol, int *fd);
char * Sock_ntop(const struct sockaddr *sa, socklen_t salen);


/* Following functions are defined in wrap_pthread.c */
void Pthread_create(pthread_t *tid, const pthread_attr_t *attr, void * (*func)(void *), void *arg);
void Pthread_join(pthread_t tid, void **status);
void Pthread_detach(pthread_t tid);
void Pthread_mutexattr_init(pthread_mutexattr_t *attr);
void Pthread_mutex_init(pthread_mutex_t *mptr, pthread_mutexattr_t *attr);
void Pthread_mutex_destroy(pthread_mutex_t *mptr);
void Pthread_mutex_lock(pthread_mutex_t *mptr);
void Pthread_mutex_unlock(pthread_mutex_t *mptr);
void Pthread_cond_init(pthread_cond_t *cptr, const pthread_condattr_t *attr);
void Pthread_cond_destroy(pthread_cond_t *cptr);
void Pthread_cond_broadcast(pthread_cond_t *cptr);
void Pthread_cond_signal(pthread_cond_t *cptr);
void Pthread_cond_wait(pthread_cond_t *cptr, pthread_mutex_t *mptr);
void Pthread_cond_timedwait(pthread_cond_t *cptr, pthread_mutex_t *mptr, const struct timespec *tsptr);
void Pthread_once(pthread_once_t *ptr, void (*func)(void));
void Pthread_key_create(pthread_key_t *key, void (*func)(void *));
void Pthread_setspecific(pthread_key_t key, const void *value);
void Pthread_attr_init(pthread_attr_t *attr);
void Pthread_attr_destroy(pthread_attr_t *attr);
void Pthread_attr_setdetachstate(pthread_attr_t *attr, int detachstate);
void Pthread_attr_setscope(pthread_attr_t *attr, int contentionscope);
void Pthread_sigmask(int how, const sigset_t *set, sigset_t *oset);


/* Following functions are defined in wrap_string.c */
void * Mdup(const void * s, size_t n);
void Mzero(void *s, size_t n);
void Mmove(void *s1, const void *s2, size_t n);
size_t Strlen(const char *s);
int Strcmp(const char *s1, const char *s2);
int Strncmp(const char *s1, const char *s2, size_t n);
int Streq(const char *s1, const char *s2);
char * Strncpy(char *s1, const char *s2, size_t n);
char * Strdup(const char *str);
int Vsnprintf(char *s, size_t n, const char *format,  va_list ap);
int Snprintf(char *s, size_t  n, const char *fmt, ...);


/* Following functions are defined in list.c */
list_t list_new(int is_sync);
void list_dispose(list_t list, void (*func)(void *));
int list_is_empty(list_t list);
int list_is_empty_opt(list_t list, int is_sync);
int list_get_nelm(list_t list);
void list_insert(list_t list, void * val);
void list_delete(list_t list, void * val, void (*func)(void *));
void list_delete_opt(list_t list, void * val, void (*func)(void *), int is_sync);
void list_enqueue(list_t list, void * val);
void * list_dequeue(list_t list);
void * list_dequeue_opt(list_t list, int is_block, int (*func)(void *, void *, void *), void * arg1, void * arg2);
void * list_find(list_t list, int (*func)(void *, void *), void * arg);
void list_map(list_t list, void (*func)(void *, void *), void * arg);
void list_sort(list_t p, int (*func)(void *, void *));
void list_swap(list_t p,  list_t q, int is_sync);


int tcp_connect(const char * host, const char * serv);
int tcp_listen(const char *host, const char *serv, socklen_t *addrlenp);


#define min(a,b)	((a) < (b) ? (a) : (b))
#define max(a,b)	((a) > (b) ? (a) : (b))



/* Following functions enables us to define ***_list structure easily.  */

#define DEFINE_LIST_STRUCTURE(name) \
struct name ## _cell { \
  name ## _t		self; \
  struct name ## _cell 	* next; \
}; \
\
typedef struct name ## _cell	* name ## _cell_t; \
\
struct name ## _list { \
  name ## _cell_t	head; \
  name ## _cell_t	tail; \
  int			nelm; \
  int			is_sync; \
  pthread_mutex_t	mp; \
  pthread_cond_t	i_cond; \
  pthread_cond_t	d_cond; \
}; \
\
typedef struct name ## _list	* name ## _list_t; 


#define DEFINE_LIST_FUNC_PROTOTYPE(name) \
name ## _list_t name ## _list_new        (int is_sync); \
void            name ## _list_dispose    (name ## _list_t list); \
int             name ## _list_is_empty   (name ## _list_t list); \
int             name ## _list_is_empty_opt(name ## _list_t list, int is_sync); \
int             name ## _list_get_nelm   (name ## _list_t list); \
void            name ## _list_insert     (name ## _list_t list, name ## _t val); \
void            name ## _list_delete     (name ## _list_t list, name ## _t val); \
void            name ## _list_delete_opt (name ## _list_t list, name ## _t val, int is_sync); \
void            name ## _list_enqueue    (name ## _list_t list, name ## _t val); \
void            name ## _list_enqueue_opt(name ## _list_t list, name ## _t val, int is_sync); \
name ## _t      name ## _list_dequeue    (name ## _list_t list); \
name ## _t      name ## _list_dequeue_opt(name ## _list_t list, int is_block, int (*func)(name ## _t, void *, void *), void * arg1, void * arg2); \
void            name ## _list_map        (name ## _list_t list, void (*func)(name ## _t, void *), void * arg); \
void	        name ## _list_sort       (name ## _list_t list, int (*func)(name ## _t, name ## _t)); \
void            name ## _list_swap       (name ## _list_t p, name ## _list_t q, int is_sync);


#define DEFINE_LIST_FUNC_BODY(name) \
name ## _list_t name ## _list_new(int is_sync){ \
  return (name ##_list_t)list_new(is_sync); \
} \
\
void name ## _list_dispose(name ## _list_t list) { \
  list_dispose((list_t)list, (func1) &name ## _dispose); \
} \
\
int name ## _list_is_empty(name ## _list_t list) { \
  return list_is_empty((list_t)list); \
} \
\
int name ## _list_is_empty_opt(name ## _list_t list, int is_sync) { \
  return list_is_empty_opt((list_t)list, is_sync); \
} \
\
int             name ## _list_get_nelm  (name ## _list_t list) { \
  return list_get_nelm((list_t)list); \
} \
\
void name ## _list_insert(name ## _list_t list, name ## _t val) { \
  list_insert((list_t)list, (void *)val); \
} \
\
void name ## _list_delete(name ## _list_t list, name ## _t val) { \
  list_delete((list_t)list, (void *)val, (func1)&name ## _dispose); \
}\
\
void name ## _list_delete_opt(name ## _list_t list, name ## _t val, int is_sync){ \
  list_delete_opt((list_t)list, (void *)val, (func1)&name ## _dispose, is_sync); \
} \
\
void name ## _list_enqueue(name ## _list_t list, name ## _t val) { \
  list_enqueue((list_t)list, (void *)val); \
} \
\
name ## _t name ## _list_dequeue(name ## _list_t list) { \
  return (name ## _t)list_dequeue((list_t)list); \
} \
\
name ## _t name ## _list_dequeue_opt(name ## _list_t list, int is_block, int (*func)(name ## _t, void *, void *), void * arg1, void * arg2) { \
  return (name ## _t)list_dequeue_opt((list_t)list, is_block, (func4)func, arg1, arg2); \
} \
\
name ## _t name ## _list_find(name ## _list_t list, int (*func)(name ## _t, void *), void * arg) { \
  return (name ## _t)list_find((list_t)list, (func2)func, arg); \
} \
\
void name ## _list_map(name ## _list_t list, void (*func)(name ## _t, void *), void * arg) { \
  list_map((list_t)list, (func3)func, arg); \
} \
void name ## _list_sort(name ## _list_t list, int (*func)(name ## _t, name ## _t)) { \
  list_sort((list_t)list, (func2)func); \
} \
\
void name ## _list_swap(name ## _list_t p, name ## _list_t q, int is_sync) {\
  list_swap((list_t)p, (list_t)q, is_sync); \
} \



#endif


  






